/*
 * Copyright (c) 2003, Adam Dunkels.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution. 
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.  
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  
 *
 * This file is part of the "ctk" console GUI toolkit for cc65
 *
 * $Id: ctk-mouse.h,v 1.1 2006/06/17 22:41:16 adamdunkels Exp $
 *
 */
#ifndef __CTK_MOUSE_H__
#define __CTK_MOUSE_H__

#include "contiki-conf.h"

#if CTK_CONF_MOUSE_SUPPORT

void ctk_mouse_init(void);

unsigned short ctk_mouse_x(void);
unsigned short ctk_mouse_y(void);

unsigned char ctk_mouse_xtoc(unsigned short x);
unsigned char ctk_mouse_ytoc(unsigned short y);

unsigned char ctk_mouse_button(void);

void ctk_mouse_hide(void);
void ctk_mouse_show(void);

#else

#define ctk_mouse_init()
#define ctk_mouse_x() 0
#define ctk_mouse_y() 0
#define ctk_mouse_xtoc(x) 0
#define ctk_mouse_ytoc(y) 0

#define ctk_mouse_button() 0

#define ctk_mouse_hide()
#define ctk_mouse_show()

#endif /* CTK_CONF_MOUSE_SUPPORT */

#endif /* __CTK_MOUSE_H__ */
